import { defineStore } from 'pinia';
import { api } from '../../boot/axios';
import { tagsToString, stringToTags } from '../../code-helper/string-helper.js';

export const useFiltersStore = defineStore('filters', {
  state: () => ({
    filters: [],
    isLoading: false,
  }),

  actions: {
    addFilter(filter) {

        const index = this.filters.findIndex(element => element.name === filter.name);

        // add filter to pinia
        if(index === -1) {
            // filter_name exists not -> add object
            this.filters.push(filter);
        } else {
            // filter_name exists -> replace object
            this.filters.splice(index, 1, filter);
        };

        // add filter to db
        if(index === -1) {
             // filter_name exists not -> add new object
             api({
                method: 'post',
                url: '/filters',
                data: {
                    name: filter.name,
                    user: filter.user ? filter.user : null,
                    poly: filter.shape ? JSON.stringify(filter.shape) : null,
                    isRectangle : filter.isRectangle,
                    tags: filter.tags ? tagsToString(filter.tags) : null,
                    dateFrom: filter.dateFrom ? filter.dateFrom : null,
                    dateTo: filter.dateTo ? filter.dateTo: null,
                    changesetStatus: filter.changesetStatus ? filter.changesetStatus : null
                }
            })
            .then((response) => {
                console.log("post create filter response: ", response.data);
            })
            .catch((error) => {
                console.error("Error on post create filter, error.message is: ", error.message);
            });
        } else {
            // filter_name exists -> only replace object
            api({
                method: 'put',
                url: `/filters/${filter.name}`,
                data: {
                    name: filter.name,
                    user: filter.user ? filter.user : null,
                    poly: filter.shape ? JSON.stringify(filter.shape) : null,
                    isRectangle : filter.isRectangle,
                    tags: filter.tags ? tagsToString(filter.tags) : null,
                    dateFrom: filter.dateFrom ? filter.dateFrom : null,
                    dateTo: filter.dateTo ? filter.dateTo : null,
                    changesetStatus: filter.changesetStatus ? filter.changesetStatus : null
                }
            })
            .then((response) => {
                console.log("put update filter response: ", response.data);
            })
            .catch((error) => {
                console.error("Error on put update filter, error.message is: ", error.message);
            });
        };
    },

    deleteFilter(filter_name) {
        // delete filter on pinia
        this.filters = this.filters.filter(element => {
            return element.name !== filter_name;
        });

        // TODO: check if element exists and only then delete it on db

        // delete filter on db
        api({
                method: 'delete',
                url: `/filters/${filter_name}`,
            })
            .then((response) => {
                console.log("delete filter response: ", response.data);
            })
            .catch((error) => {
                console.error("Error on delete filter, error.message is: ", error.message);
            });
    },

    getFilter(filter_name) {
         return this.filters.find(element => element.name === filter_name);
    },

    getAllFilters() {
        this.isLoading = true;
        api({
            method: 'get',
            url: '/filters',
        })
        .then((response) => {
            this.isLoading = false;
            console.log("get all filters response: ", response.data);

            const tmp = response.data;
            for(let i=0; i<tmp.length; i++) {

                if(tmp[i].shape !== null) {
                    tmp[i].shape = JSON.parse(JSON.parse(tmp[i].shape));
                }

                if(tmp[i].tags !== null) {
                    tmp[i].tags = stringToTags(tmp[i].tags);
                }
            }
            this.filters = tmp;
        })
        .catch((error) => {
            console.error("Error on get all filters, error.message is: ", error.message);
        });
    },
  },
});
